/** @par LICENCE
* @verbatim
 Copyright (C) 2004,2005  ADDI-DATA GmbH for the source code and the documentation.

        ADDI-DATA GmbH
        Dieselstrasse 3
        D-77833 Ottersweier
        Tel: +19(0)7223/9493-0
        Fax: +49(0)7223/9493-92
        http://www.addi-data-com
        info@addi-data.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You shoud find the complete GPL in the COPYING file accompanying
    this source code.
* @endverbatim
*/

#include "apci2016-private.h"

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
#include <linux/sysfs.h>
#endif

//BEGIN JK 06.12.2004: Module informatIon
MODULE_LICENSE("GPL");
MODULE_AUTHOR("ADDI-DATA GmbH <info@addi-data.com>");
MODULE_DESCRIPTION("APCI-2016 IOCTL driver");

EXPORT_NO_SYMBOLS;


//------------------------------------------------------------------------------

atomic_t apci2016_count = ATOMIC_INIT(0);
unsigned int apci2016_majornumber = 0;

//------------------------------------------------------------------------------
static int __devinit apci2016_probe_one(struct pci_dev *dev,const struct pci_device_id *ent);
static void __devexit apci2016_remove_one(struct pci_dev *dev);

//------------------------------------------------------------------------------
/** The ID table is an array of struct pci_device_id ending with a all-zero entry.
Each entry consists of:

	- vendor, device	Vendor and device ID to match (or PCI_ANY_ID)
	- subvendor,	Subsystem vendor and device ID to match (or PCI_ANY_ID)
	- subdevice class,		Device class to match. The class_mask tells which bits
	- class_mask	of the class are honored during the comparison.
	- driver_data	Data private to the driver.
*/

static struct pci_device_id apci2016_pci_tbl[] __initdata = {
	{
		APCI2016_BOARD_VENDOR_ID,
		APCI2016_BOARD_DEVICE_ID,
		PCI_ANY_ID,
		PCI_ANY_ID,
		0,
		0,
		0
	},

	{ 0 },	/* terminate list */
};

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
static struct class * apci2016_class = NULL;
#endif

MODULE_DEVICE_TABLE (pci, apci2016_pci_tbl);

//----------------------------------------------------------------------------------------------------------------------
static struct pci_driver apci2016_pci_driver = {
	.name		= __DRIVER_NAME,
	.probe		= apci2016_probe_one,
	.remove		= __devexit_p(apci2016_remove_one),
	.id_table	= apci2016_pci_tbl,
};
//----------------------------------------------------------------------------------------------------------------------


/* Prototypes of the used functions
 */
int APCI2016_open   (struct inode *inode, struct file *filp);
int APCI2016_release(struct inode *inode, struct file *filp);
int APCI2016_ioctl  (struct        inode *inode,
                     struct        file *filp,
		     unsigned int  cmd,
		     unsigned long arg);



//------------------------------------------------------------------------------
static struct file_operations apci2016_fops =
{
	.ioctl		= apci2016_ioctl_lookup,
	.open		= apci2016_open_lookup,
	.release	= apci2016_release_lookup,
};

//------------------------------------------------------------------------------
/** when module is unloaded, stop all board activities (cf interrupt)*/
static void apci2016_stop_board(struct pci_dev * pdev)
{

}

//------------------------------------------------------------------------------
/** checks if the given IO port region is free.
 *
 *
 *
 * */
static int apci2016_check_region(struct pci_dev * pdev, unsigned int __BAR)
{
	/* The definition of struct resource  */
	if (__BAR > 4)
	{
		printk(KERN_WARNING "%s: incorrect BAR number %d. Check your code.\n",__DRIVER_NAME, __BAR);
		return 1;
	}

	/* 2.4 only : check_region is deprecated in 2.6 */
	#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
		if ( check_region(pdev->resource[__BAR].start, ((pdev->resource[__BAR].end) - (pdev->resource[__BAR].start)) ) )
		{
			printk(KERN_WARNING "%s: BAR %d (%lX) is already in use\n",__DRIVER_NAME,__BAR,pdev->resource[__BAR].start);
			return 1;
		}
	#endif // 2.4
	return 0;
}


//------------------------------------------------------------------------------
/** called when a new card is detected.
 *
 * Historically - in former ADDI-DATA drivers data about a board has been stored
 * in a static structure.
 * This led to huge duplication of information since most of these information
 * are already present in the pci_dev struct.
 *
 * Now the apci2016_str_BoardInformations contains a field "dev", a pointer to
 * the OS PCI device structure.
 *
 *  */
static int __devinit apci2016_probe_one(struct pci_dev *dev,
				     const struct pci_device_id *ent)
{
	{
	    int ret = pci_enable_device(dev);
	    if (ret)
	    {
	            printk(KERN_ERR "%s: pci_enable_device failed\n",__DRIVER_NAME);
	            return ret;
	    }
	}

	/* check if BAR 0 is free */
	if ( apci2016_check_region(dev,0) )
	{
		return -EBUSY;
	}
	/* check if BAR 1 is free */
	if ( apci2016_check_region(dev,1) )
	{
		return -EBUSY;
	}
	/* check if BAR 2 is free */
	if ( apci2016_check_region(dev,2) )
	{
		return -EBUSY;
	}

	/* allocate a new data structure for the board */
	{
		struct apci2016_str_BoardInformations * newboard_data = NULL;
		newboard_data = kmalloc( sizeof( struct apci2016_str_BoardInformations) , GFP_ATOMIC);
		if (!newboard_data)
		{
			printk(KERN_CRIT "Can't allocate memory for new board %s\n",pci_name(dev));
			return -ENOMEM;
		}

		/* link standard data structure to the board's private one */
		pci_set_drvdata(dev,newboard_data);
	}

	/* initialise board's private data */
	apci2016_init_priv_data(APCI2016_PRIVDATA(dev));

	{
		/* increase the global board count */
		atomic_inc(&apci2016_count);
		printk(KERN_INFO "%s: board %s managed (minor number will be %d) \n",__DRIVER_NAME, pci_name(dev), atomic_read(&apci2016_count)-1);
	}

	/* allocate BAR 0 to 2 memory regions */
	{
	    int ret = pci_request_regions(dev,__DRIVER_NAME);
	    if (ret)
	    {
	            printk(KERN_ERR "%s: pci_request_regions failed\n",__DRIVER_NAME);
	            /* free all allocated ressources here*/
	            kfree(APCI2016_PRIVDATA(dev));
	            return ret;
	    }
	}

	/* create /proc */
	apci2016_proc_create_device(dev, atomic_read(&apci2016_count)-1);

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
	while(1)
	{
	#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,26)
		struct class_device *cdev;
	#else
		struct device *cdev;
	#endif

		int minor = (atomic_read(&apci2016_count)-1);

		/* don't execute if class not exists */
		if (IS_ERR(apci2016_class))
			break;

	#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,26)
		cdev = class_device_create (apci2016_class, NULL, MKDEV(apci2016_majornumber, minor), NULL, "%s_%d", __DRIVER_NAME, minor);
	#elif LINUX_VERSION_CODE < KERNEL_VERSION(2,6,27)
		cdev = device_create (apci2016_class, NULL, MKDEV(apci2016_majornumber, minor), "%s_%d", __DRIVER_NAME, minor);
	#else
		cdev = device_create (apci2016_class, NULL, MKDEV(apci2016_majornumber, minor), NULL, "%s_%d", __DRIVER_NAME, minor);
	#endif

		if (IS_ERR(cdev))
		{
			printk (KERN_WARNING "%s: class_device_create error\n", __DRIVER_NAME);
		}
		break;
	}
#endif

	return 0;
}


//------------------------------------------------------------------------------
/** This function registers the driver with register_chrdev.
 *
 * @todo For kernel 2.6, use udev
 *  */

static int apci2016_register(void)
{

	//Registration of driver
	apci2016_majornumber = register_chrdev(0, __DRIVER_NAME, &apci2016_fops);

	if (apci2016_majornumber < 0)
	{
		printk (KERN_ERR "%s: register_chrdev returned %d ... aborting\n",__DRIVER_NAME,apci2016_majornumber);
		return -ENODEV;
	}
	return 0;
}

//------------------------------------------------------------------------------
/** Called when module loads. */
static int __init apci2016_init(void)
{

	/* KERNEL 2.4 --------------------------------------------- */
	#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
		/* Detection if an PCI Bus is present */
		if(!pci_present())
		{
			printk(KERN_WARNING "%s: NO PCI BUS.\n", __DRIVER_NAME);
			return -ENODEV;
		}
	#endif // 2.4

	if( apci2016_register() )
	{
		return -ENODEV;
	}

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
	apci2016_class = class_create (THIS_MODULE, __DRIVER_NAME);
	if (IS_ERR(apci2016_class))
 	{
 		printk (KERN_WARNING "%s: class_create error\n",__DRIVER_NAME );
 	}
#endif

	/* registred, now create root /proc entry */
	apci2016_proc_init();

	printk(KERN_INFO "%s: loaded\n",__DRIVER_NAME);

	/* now, subscribe to PCI bus subsystem  */
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,18)
	/** @note The module will load anyway even if the call to pci_module_init() fails */
	if ( pci_module_init (&apci2016_pci_driver) != 0)
#else
	if ( pci_register_driver (&apci2016_pci_driver) != 0)
#endif
	{
		printk("%s: no device found\n",__DRIVER_NAME);
		apci2016_pci_driver.name = "";
	}

	return 0;
}

//-------------------------------------------------------------------
/** a card is removed */
static void __devexit apci2016_remove_one(struct pci_dev *dev)
{
	/* stop board activities */
	apci2016_stop_board(dev);

	/* do system-dependant thing we don't really want to know of :) */
	pci_disable_device(dev);

	/* deallocate BAR 0 to 2 memory regions */
	pci_release_regions(dev);

	/* free private device data */
	kfree(APCI2016_PRIVDATA(dev));

	/* delete associated /proc entry */
	apci2016_proc_release_device(dev);

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
        while(1)
        {
        	int minor = (atomic_read(&apci2016_count)-1);

			/* don't execute if class not exists */
			if(IS_ERR(apci2016_class))
					break;

			#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,26)
				class_device_destroy (apci2016_class, MKDEV(apci2016_majornumber, minor));
			#else
				device_destroy (apci2016_class, MKDEV(apci2016_majornumber, minor));
			#endif

            break;
        }
#endif

	/* decrement global board count*/
	atomic_dec(&apci2016_count);

	printk("%s: board %s unloaded \n",__DRIVER_NAME,pci_name(dev));

}

//------------------------------------------------------------------------------
/** This function unregisters the driver with unregister_chrdev.
 *
 * @todo For kernel 2.6, use udev
 *  */

static void apci2016_unregister(void)
{
	unregister_chrdev(apci2016_majornumber, __DRIVER_NAME);
}

//-------------------------------------------------------------------
/** Called when module is unloaded. */
static void __exit apci2016_exit(void)
{

	/* Cleans the module from the kernel */
	if (apci2016_pci_driver.name[0])
	{
		pci_unregister_driver (&apci2016_pci_driver);
	}

	apci2016_unregister();
	apci2016_proc_release();

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
        /* don't execute if class not exists */
        if ( !IS_ERR(apci2016_class) )
                class_destroy (apci2016_class);
#endif
}
//------------------------------------------------------------------------------
module_exit(apci2016_exit);
module_init(apci2016_init);
